@@ -28,7 +28,6 @@ Epson FB II ECF driver
 """
 
 import datetime
-#import time
 import struct
 from decimal import Decimal
 
@@ -42,9 +41,9 @@ from stoqdrivers.interfaces import ICouponPrinter
 from stoqdrivers.exceptions import (DriverError, PrinterError, CommandError,
                                     CommandParametersError, OutofPaperError,
                                     CancelItemError, AlmostOutofPaper,
-                                    ItemAdditionError)
+                                    ItemAdditionError, CouponOpenError,
+                                    CouponNotOpenError)
 from stoqdrivers.enum import TaxType, UnitType
-#from stoqdrivers.printers.capabilities import Capability
 from stoqdrivers.printers.base import BaseDriverConstants
 from stoqdrivers.translation import stoqdrivers_gettext
 
@@ -57,14 +56,16 @@ FLD = '\x1c'
 # Special protocol chars that need to be escaped when communicating
 SPECIAL_CHARS = [ESC, STX, ETX, FLD, '\x1a', '\x1d', '\x1e', '\x1f']
 
+
 def escape(string):
     for i in SPECIAL_CHARS:
-        string = string.replace(i, ESC+i)
+        string = string.replace(i, ESC + i)
     return string
 
+
 def unescape(string):
     for i in SPECIAL_CHARS:
-        string = string.replace(ESC+i, i)
+        string = string.replace(ESC + i, i)
     return string
 
 FIRST_COMMAND_ID = 0x81
@@ -83,6 +84,7 @@ _ = stoqdrivers_gettext
 
 log = Logger('stoqdrivers.epson')
 
+
 class Reply(object):
 
     #
@@ -104,6 +106,9 @@ class Reply(object):
         '0304': (OutofPaperError(_("Out of paper."))),
         '0305': (AlmostOutofPaper(_("Almost out of paper."))),
 
+        '0801': (CommandError(_("Invalid command with closed "
+                                "fiscal journey."))),
+
         '090C': (DriverError(_("Payment method not defined."))),
         '090F': (ItemAdditionError(_("Tax not found."))),
         '0910': (ItemAdditionError(_("Invalid tax."))),
@@ -111,6 +116,7 @@ class Reply(object):
         '0A12': (CancelItemError(_("It was not possible cancel the last "
                                    "fiscal coupon."))),
         '0A15': (PrinterError(_("Requires CDC cancellation."))),
+        '0A16': (CancelItemError(_("Invalid item number in fiscal coupon"))),
         '0E0A': (PrinterError(_("Last non-fiscal coupon not found."))),
         '0E0B': (PrinterError(_("Payment method not found."))),
     }
@@ -149,7 +155,6 @@ class Reply(object):
         r = self.pop(2)
         self.reply_status = '%02X%02X' % (ord(r[0]), ord(r[1]))
 
-
         assert self.pop() == FLD, 'FLD 3'
         # reserved
         self.pop()
@@ -199,11 +204,11 @@ class FBIIConstants(BaseDriverConstants):
     # The 'unit' field is mandatory in ECF.
     # If unit is EMPTY, set the value 'UN' to avoid errors.
     _constants = {
-        UnitType.WEIGHT:      'Kg',
-        UnitType.METERS:      'm',
-        UnitType.LITERS:      'Lt',
-        UnitType.EMPTY:       'UN',
-        }
+        UnitType.WEIGHT: 'Kg',
+        UnitType.METERS: 'm',
+        UnitType.LITERS: 'Lt',
+        UnitType.EMPTY: 'UN',
+    }
 
 
 #
@@ -223,7 +228,7 @@ class FBII(SerialBase):
     def __init__(self, port, consts=None):
         SerialBase.__init__(self, port)
         self._consts = consts or FBIIConstants
-        self._command_id = FIRST_COMMAND_ID-1 #0x80
+        self._command_id = FIRST_COMMAND_ID - 1  # 0x80
         self._reset()
 
     def setup(self):
@@ -243,7 +248,7 @@ class FBII(SerialBase):
 
     def _get_next_command_id(self):
         if self._command_id == 256:
-            self._command_id = FIRST_COMMAND_ID-1
+            self._command_id = FIRST_COMMAND_ID - 1
 
         self._command_id += 1
         return chr(self._command_id)
@@ -260,7 +265,7 @@ class FBII(SerialBase):
                 frame += FLD + escape(i)
 
         command_id = self._get_next_command_id()
-        package = STX +  command_id + frame + ETX
+        package = STX + command_id + frame + ETX
 
         checksum = sum([ord(d) for d in package])
         return package + '%04X' % checksum
@@ -304,7 +309,7 @@ class FBII(SerialBase):
         ack = self.read(1)
         if not ack:
             raise DriverError(_("Timeout communicating with fiscal "
-                                        "printer"))
+                                "printer"))
         assert ack == ACK, repr(ack)
 
         reply = self._read_reply()
@@ -459,6 +464,9 @@ class FBII(SerialBase):
         return checked_coupon != CLOSED_COUPON
 
     def coupon_open(self):
+        coupon_status = self._get_coupon_status()
+        if coupon_status == OPENED_FISCAL_COUPON:
+            raise CouponOpenError(_("Coupon already open"))
         self._send_command('0A01', '0000', '', '')
 
     def _cancel_fiscal_coupon(self):
@@ -478,6 +486,10 @@ class FBII(SerialBase):
                         quantity=Decimal("1.0"), unit=UnitType.EMPTY,
                         discount=Decimal("0.0"), markup=Decimal("0.0"),
                         unit_desc=""):
+        coupon_status = self._get_coupon_status()
+        if coupon_status != OPENED_FISCAL_COUPON:
+            raise CouponNotOpenError(_("Coupon is not open"))
+
         if unit == UnitType.CUSTOM:
             unit = unit_desc
         else:
@@ -485,11 +497,26 @@ class FBII(SerialBase):
         qtd = int(quantity * self._decimals_quantity)
         value = int(price * self._decimals_price)
         st = taxcode
+        # Register the sale item
         reply = self._send_command('0A02', '0000', code, description,
                                    str(qtd), unit, str(value), st)
         id = reply.fields[0]
+        if discount:
+            self.apply_discount(id, discount)
+        elif markup:
+            self.apply_markup(id, markup)
         return int(id)
 
+    def apply_discount(self, id, discount):
+        value = int(discount * Decimal('1e2'))
+        # Register the discount on the item
+        self._send_command('0A04', '0004', str(value))
+
+    def apply_markup(self, id, markup):
+        value = int(markup * Decimal('1e2'))
+        # Register the markup on the item
+        self._send_command('0A04', '0005', str(value))
+
     def coupon_cancel_item(self, item_id):
         self._send_command('0A18', '0004', str(item_id))
 
@@ -542,6 +569,9 @@ class FBII(SerialBase):
                                self._customer_address[41:79])
             self._reset()
 
+        if message:
+            self._print_promotional_message(message)
+
         reply = self._send_command('0A06', '0001')
         self._send_command('0702', '0000')
         return int(reply.fields[0])
@@ -564,7 +594,7 @@ class FBII(SerialBase):
             if line == '':
                 self._print_line(line)
             for pos in range(0, len(line), 56):
-                data = line[pos:pos+56]
+                data = line[pos:pos + 56]
                 self._print_line(data)
 
     def _print_line(self, line):
@@ -576,6 +606,17 @@ class FBII(SerialBase):
         line = line.strip('\r')
         self._send_command('0E02', '0000', line)
 
+    def _print_promotional_message(self, message):
+        msg = []
+        # Epson accepts only 56 chars per line. Lets truncate each line
+        for line in message.split('\n'):
+            msg.append(line[:56])
+
+        # We must send 8 lines to fiscal printer, even if they are empty.
+        while len(msg) < 8:
+            msg.append('')
+        self._send_command('0A22', '0000', *msg[:8])
+
     #
     #   General information
     #
@@ -591,7 +632,7 @@ class FBII(SerialBase):
         fields = reply.fields
         if len(fields) > 2:
             for i in range(0, len(fields), 3):
-                name, value = fields[i], fields[i+1]
+                name, value = fields[i], fields[i + 1]
 
                 if name.startswith('T'):
                     type = TaxType.CUSTOM
@@ -604,9 +645,9 @@ class FBII(SerialBase):
 
         constants.extend([
             (TaxType.SUBSTITUTION, 'F', None),
-            (TaxType.EXEMPTION,    'I', None),
-            (TaxType.NONE,         'N', None),
-            ])
+            (TaxType.EXEMPTION, 'I', None),
+            (TaxType.NONE, 'N', None),
+        ])
 
         return constants
 
@@ -615,14 +656,14 @@ class FBII(SerialBase):
 
         for i in range(20):
             try:
-                reply = self._send_command('050D', '0000', '%d' % (i+1))
+                reply = self._send_command('050D', '0000', '%d' % (i + 1))
             except DriverError, e:
-                if e.code == 0x090C: # Tipo de pagamento não definido
+                if e.code == 0x090C:  # Tipo de pagamento não definido
                     continue
                 else:
                     raise
             name, vinculado = reply.fields
-            methods.append(('%d' % (i+1), name.strip()))
+            methods.append(('%d' % (i + 1), name.strip()))
 
         return methods
 
@@ -634,7 +675,7 @@ class FBII(SerialBase):
             customer_id=Capability(max_len=28),
             customer_name=Capability(max_len=30),
             customer_address=Capability(max_len=79),
-            )
+        )
 
     def get_constants(self):
         return self._consts
@@ -672,7 +713,7 @@ class FBII(SerialBase):
 
         taxes = []
         for i in range(20):
-            reg = tax_codes[i*3:i*3+3]
+            reg = tax_codes[i * 3:i * 3 + 3]
             if not reg:
                 continue
 
@@ -680,7 +721,7 @@ class FBII(SerialBase):
             if name.startswith('T'):
                 tax_type = 'ICMS'
             else:
-                tax_type =  'ISS'
+                tax_type = 'ISS'
 
             sold = reg[2]
             value = reg[1]
@@ -749,12 +790,11 @@ class FBII(SerialBase):
         self._define_tax_code("0900", service=True)
 
 
-
 if __name__ == '__main__':
     from stoqdrivers.serialbase import SerialPort
     #port = Serial('/dev/pts/5')
     port = SerialPort('/dev/ttyUSB0')
-    p  = FBII(port)
+    p = FBII(port)
 
 #    p._send_command('0754', '0000') Desativar corte do papel
 
