@@ -49,6 +49,7 @@ from stoqdrivers.translation import stoqdrivers_gettext
 
 _ = stoqdrivers_gettext
 
+
 class CouponItem:
     def __init__(self, id, quantity, value):
         self.id, self.quantity, self.value = id, quantity, value
@@ -56,13 +57,14 @@ class CouponItem:
     def get_total_value(self):
         return self.quantity * self.value
 
+
 class FakeConstants(BaseDriverConstants):
     _constants = {
-        UnitType.WEIGHT:      'F',
-        UnitType.METERS:      'G ',
-        UnitType.LITERS:      'H',
-        UnitType.EMPTY:       'I',
-        }
+        UnitType.WEIGHT: 'F',
+        UnitType.METERS: 'G ',
+        UnitType.LITERS: 'H',
+        UnitType.EMPTY: 'I',
+    }
 
     _payment_constants = {
         PaymentMethodType.MONEY: 'M',
@@ -72,7 +74,7 @@ class FakeConstants(BaseDriverConstants):
         PaymentMethodType.DEBIT_CARD: 'D',
         PaymentMethodType.FINANCIAL: 'F',
         PaymentMethodType.GIFT_CERTIFICATE: 'G',
-        }
+    }
 
     _payment_descriptions = {
         'M': 'Dinheiro',
@@ -86,16 +88,18 @@ class FakeConstants(BaseDriverConstants):
 
     _tax_constants = [
         (TaxType.SUBSTITUTION, 'TS', None),
-        (TaxType.EXEMPTION,    'TE', None),
-        (TaxType.NONE,         'TN', None),
-        (TaxType.CUSTOM,       'T1', Decimal(18)),
-        (TaxType.CUSTOM,       'T2', Decimal(12)),
-        (TaxType.CUSTOM,       'T3', Decimal(5)),
-        (TaxType.SERVICE,      'S0', Decimal(3)),
-        ]
+        (TaxType.EXEMPTION, 'TE', None),
+        (TaxType.NONE, 'TN', None),
+        (TaxType.CUSTOM, 'T1', Decimal(18)),
+        (TaxType.CUSTOM, 'T2', Decimal(12)),
+        (TaxType.CUSTOM, 'T3', Decimal(5)),
+        (TaxType.SERVICE, 'S0', Decimal(3)),
+    ]
+
 
 class OutputWindow(gtk.Window):
     columns = 60
+
     def __init__(self, printer):
         self._printer = printer
         gtk.Window.__init__(self)
@@ -301,10 +305,7 @@ class Simple(object):
                         discount=Decimal("0.0"),
                         surcharge=Decimal("0.0"), unit_desc=""):
         self._check()
-        if code not in ['2368694135945', '6234564656756', '6985413595971',
-                        '2692149835416', '1595843695465', '8596458216412',
-                        '9586249534513', '7826592136954', '5892458629421',
-                        '1598756984265', '1598756984265']:
+        if code not in ["{:02d}".format(x) for x in range(1, 11)]:
             # Allow deliveries as well
             if code != '' and taxcode != 'S0':
                 raise ItemAdditionError(
@@ -331,7 +332,7 @@ class Simple(object):
         elif self.is_coupon_totalized:
             raise CancelItemError(_("The coupon is already totalized, "
                                     "you can't cancel items anymore."))
-        item = self._items.pop(item_id)
+        self._items.pop(item_id)
         self.output.feed('cancel_item %r\n' % (item_id, ))
 
     def coupon_cancel(self):
@@ -383,7 +384,7 @@ class Simple(object):
         self.payments_total += value
         self.has_payments = True
         self.output.feed('  %s - %s\n' % (
-                self._consts._payment_descriptions[payment_method], value))
+            self._consts._payment_descriptions[payment_method], value))
         return self.totalized_value - self.payments_total
 
     def coupon_close(self, message=''):
@@ -402,6 +403,8 @@ class Simple(object):
         if troco:
             self.output.feed('Troco: %0.2f\n' % troco)
         self.output.feed_line()
+        if message:
+            self.output.feed(message)
         self.output.feed('\n')
         self._reset_flags()
         return 0
@@ -488,7 +491,7 @@ class Simple(object):
     def get_payment_constants(self):
         self._check()
         return [(key, value) for key, value in
-                        self._consts._payment_descriptions.items()]
+                self._consts._payment_descriptions.items()]
 
     def get_port(self):
         self._check()
@@ -560,11 +563,8 @@ class Simple(object):
     def get_firmware_version(self):
         return '1.1.1'
 
-
     #
     # IChequePrinter implementation
     #
-
     def print_cheque(self, value, thirdparty, city, date=None):
         return
-
